#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from .job import Job
from .drmaa2_object_descriptors import Drmaa2Descriptor

from .log_manager import LogManager


class Drmaa2JobListDescriptor(Drmaa2Descriptor):
    """ A descriptor for drmaa2_j_list fields. """

    logger = LogManager.get_instance().get_logger('Drmaa2JobListDescriptor')

    def __init__(self, name):
        Drmaa2Descriptor.__init__(self, name)

    def __get__(self, obj, type=None):
        if not obj:
            return None
        value_list = []
        if obj._struct is not None:
            ctypes_list = getattr(obj._struct.contents, self.name)
            value_list = Job.to_py_job_list(ctypes_list)
        return value_list

    def __set__(self, obj, value_list):
        # Always used as read-only descriptor.
        return



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

import atexit
from ctypes import POINTER

from .drmaa2_ctypes import drmaa2_callback
from .byte_string import ByteString
from .drmaa2_constants import Event
from .drmaa2_constants import JobState
from .drmaa2_object import Drmaa2Object
from .drmaa2_exceptions import InvalidArgument

from .log_manager import LogManager
from .exception_mapper import ExceptionMapper


class Notification(Drmaa2Object):
    """ High-level DRMAA2 notification class. """

    event = Drmaa2Object.EnumDescriptor('event', Event)
    """ Event number (Event). """
    session_name = Drmaa2Object.StringDescriptor('sessionName')
    """ Session name (str). """
    job_id = Drmaa2Object.StringDescriptor('jobId')
    """ Job id (str). """
    job_state = Drmaa2Object.EnumDescriptor('jobState', JobState)
    """ Job state (JobState). """
    implementation_specific = Drmaa2Object.ImplSpecDescriptor('implementationSpecific')
    """ Implementation specific dictionary ({str:str}). """

    logger = LogManager.get_instance().get_logger('Notification')

    def __init__(self, notification):
        """ 
        Constructor. 
        """
        Drmaa2Object.__init__(self)
        if isinstance(notification, POINTER(drmaa2_notification)):
            self._struct = POINTER(drmaa2_notification)()
            self._struct.contents = drmaa2_notification()
            self.event = getattr(notification.contents, 'event')
            self.job_id = ByteString(getattr(notification.contents, 'jobId').value).decode()
            self.session_name = ByteString(getattr(notification.contents, 'sessionName').value).decode()
            self.job_state = getattr(notification.contents, 'jobState')
            self.implementation_specific = getattr(notification.contents, 'implementationSpecific')
        else:
            raise InvalidArgument('Invalid argument: %s' % str(notification))
        self._read_only = True

    def __del__(self):
        pass

    @classmethod
    def get_implementation_specific_keys(cls):
        """
        Retrieve list of implementation-specific keys.

        :returns: String list of implementation-specific keys.

        >>> print(Notification.get_implementation_specific_keys())
        []
        """
        if cls.implementation_specific_keys is None:
            cls.implementation_specific_keys = cls.to_py_string_list(
                cls.get_drmaa2_library().drmaa2_notification_impl_spec())
        return cls.implementation_specific_keys

    @classmethod
    def event_callback(cls, callback):
        """ Wrap actual python callback. """

        def wrapper(notification_ptr):
            notification = Notification(notification_ptr.contents)
            callback(notification)
            cls.get_drmaa2_library().drmaa2_notification_free(notification_ptr)

        return wrapper

    @classmethod
    def register_event_notification(cls, callback):
        """
        Register callback for event notifications.

        :param callback: Callback function.
        :type callback: Function pointer.

        >>> def event_callback(notification):         
        ...     print('Event %s for job %s' % (notification.event, notification.job_id)) 
        >>> Notification.register_event_notification(event_callback)
        """
        callback_p = drmaa2_callback(cls.event_callback(callback))
        cls.logger.debug("Registering notification callback {}".format(callback_p))
        ExceptionMapper.check_status_code(cls.get_drmaa2_library().drmaa2_register_event_notification(callback_p))
        atexit.register(unregister_event_notification)

    @classmethod
    def unregister_event_notification(cls):
        """
        Unregister callback for event notifications.

        >>> Notification.unregister_event_notification()
        """
        callback_p = drmaa2_callback()
        cls.logger.debug("Unregistering notification callback {}".format(callback_p))
        ExceptionMapper.check_status_code(cls.get_drmaa2_library().drmaa2_register_event_notification(callback_p))



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from ctypes import POINTER
from ctypes import pointer

from .drmaa2_constants import Os
from .drmaa2_constants import Cpu
from .drmaa2_ctypes import drmaa2_rtemplate
from .drmaa2_ctypes import drmaa2_string
from .drmaa2_object import Drmaa2Object
from .drmaa2_exceptions import InvalidArgument


class ReservationTemplate(Drmaa2Object):
    """ High-level DRMAA2 reservation template class. """

    reservation_name = Drmaa2Object.StringDescriptor('reservationName')
    """ Reservation name (str). """
    start_time = Drmaa2Object.TimeDescriptor('startTime')
    """ Reservation start time (datetime). """
    end_time = Drmaa2Object.TimeDescriptor('endTime')
    """ Reservation end time (datetime). """
    duration = Drmaa2Object.LongLongDescriptor('duration')
    """ Reservation duration (long). """
    min_slots = Drmaa2Object.LongLongDescriptor('minSlots')
    """ Minimum number of slots required by the job (long). """
    max_slots = Drmaa2Object.LongLongDescriptor('maxSlots')
    """ Maximum number of slots required by the job (long). """
    job_category = Drmaa2Object.StringDescriptor('jobCategory')
    """ Reservation category (str). """
    users_acl = Drmaa2Object.StringListDescriptor('usersACL')
    """ Arguments for the job executable ([str]). """
    candidate_machines = Drmaa2Object.StringListDescriptor('candidateMachines')
    """ List of machines that are candidate for running the job ([str]). """
    min_phys_memory = Drmaa2Object.LongLongDescriptor('minPhysMemory')
    """ Minimum physical memory needed for the job (long). """
    machine_os = Drmaa2Object.EnumDescriptor('machineOS', Os)
    """ OS required for the job (Os). """
    machine_arch = Drmaa2Object.EnumDescriptor('machineArch', Cpu)
    """ Machine arch required for the job (Cpu). """
    implementation_specific = Drmaa2Object.ImplSpecDescriptor('implementationSpecific')
    """ Implementation specific dictionary ({str:str}). """

    def __init__(self, template={}):
        """ 
        Constructor. 

        :param template: Input structure representing the object; this structure is typically a dictionary, but it can also be a low-level drmaa2_rtemplate struct.
        :type template: dict 

        :raises InvalidArgument: in case of an invalid input argument.
        :raises Drmaa2Exception: for all other errors.

        >>> rt = ReservationTemplate({'reservation_name' : 'res-01', 'users_acl' : ['user1', 'user2']})
        >>> print(rt.reservation_name)
        res-01
        >>> rt.min_slots = 2
        """
        Drmaa2Object.__init__(self)
        if isinstance(template, dict):
            self._struct = self.get_drmaa2_library().drmaa2_rtemplate_create()
            self.init_impl_spec_key_values()
            self.from_dict(template)
        elif isinstance(template, POINTER(drmaa2_rtemplate)):
            self._struct = template
        else:
            raise InvalidArgument('Invalid argument: %s' % str(template))

    def __del__(self):
        setattr(self._struct.contents, 'reservationName', drmaa2_string())
        setattr(self._struct.contents, 'jobCategory', drmaa2_string())
        self.get_drmaa2_library().drmaa2_rtemplate_free(pointer(self._struct))

    @classmethod
    def get_implementation_specific_keys(cls):
        """
        Retrieve list of implementation-specific keys.

        :returns: String list of implementation-specific keys.

        >>> print(ReservationTemplate.get_implementation_specific_keys())
        ['uge_rt_native']
        """
        if cls.implementation_specific_keys is None:
            cls.implementation_specific_keys = cls.to_py_string_list(
                cls.get_drmaa2_library().drmaa2_rtemplate_impl_spec())
        return cls.implementation_specific_keys



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from ctypes import POINTER
from ctypes import cast

from .byte_string import ByteString
from .drmaa2_constants import Cpu
from .drmaa2_constants import Os
from .drmaa2_ctypes import drmaa2_machineinfo
from .drmaa2_object import Drmaa2Object
from .drmaa2_exceptions import InvalidArgument

from .drmaa2_version_descriptor import Drmaa2VersionDescriptor


class MachineInfo(Drmaa2Object):
    """ High-level DRMAA2 machine info class. """

    name = Drmaa2Object.StringDescriptor('name')
    """ Machine name (str). """
    available = Drmaa2Object.BoolDescriptor('available')
    """ Available flag (bool). """
    sockets = Drmaa2Object.LongLongDescriptor('sockets')
    """ Number of sockets (long). """
    cores_per_socket = Drmaa2Object.LongLongDescriptor('coresPerSocket')
    """ Number of cores per socket (long). """
    threads_per_core = Drmaa2Object.LongLongDescriptor('threadsPerCore')
    """ Number of threads per core (long). """
    load = Drmaa2Object.FloatDescriptor('load')
    """ Machine load (float). """
    phys_memory = Drmaa2Object.LongLongDescriptor('physMemory')
    """ Physical memory (long). """
    virt_memory = Drmaa2Object.LongLongDescriptor('virtMemory')
    """ Virtual memory (long). """
    machine_arch = Drmaa2Object.EnumDescriptor('machineArch', Cpu)
    """ Architecture (Cpu). """
    machine_os_version = Drmaa2VersionDescriptor('machineOSVersion')
    """ Operating system version (Version). """
    machine_os = Drmaa2Object.EnumDescriptor('machineOS', Os)
    """ Operating system (Os). """
    implementation_specific = Drmaa2Object.ImplSpecDescriptor('implementationSpecific')
    """ Implementation specific dictionary ({str:str}). """

    def __init__(self, machine_info):
        """ 
        Constructor. 

        :param machine_info: Low-level drmaa2_machineinfo struct.
        :type machine_info: drmaa2_machineinfo
        """
        Drmaa2Object.__init__(self)
        if isinstance(machine_info, POINTER(drmaa2_machineinfo)):
            self._struct = POINTER(drmaa2_machineinfo)()
            self._struct.contents = drmaa2_machineinfo()
            self.name = ByteString(getattr(machine_info.contents, 'name').value).decode()
            self.available = getattr(machine_info.contents, 'available')
            self.sockets = getattr(machine_info.contents, 'sockets')
            self.cores_per_socket = getattr(machine_info.contents, 'coresPerSocket')
            self.threads_per_core = getattr(machine_info.contents, 'threadsPerCore')
            self.load = getattr(machine_info.contents, 'load')
            self.phys_memory = getattr(machine_info.contents, 'physMemory')
            self.virt_memory = getattr(machine_info.contents, 'virtMemory')
            self.machine_arch = getattr(machine_info.contents, 'machineArch')
            self.machine_os_version = getattr(machine_info.contents, 'machineOSVersion')
            self.machine_os = getattr(machine_info.contents, 'machineOS')
            self.implementation_specific = getattr(machine_info.contents, 'implementationSpecific')
        else:
            raise InvalidArgument('Invalid argument: %s' % str(machine_info))
        self._read_only = True

    def __del__(self):
        pass

    @classmethod
    def get_implementation_specific_keys(cls):
        """
        Retrieve list of implementation-specific keys.

        :returns: String list of implementation-specific keys.

        >>> print(MachineInfo.get_implementation_specific_keys())
        []
        """
        if cls.implementation_specific_keys is None:
            cls.implementation_specific_keys = cls.to_py_string_list(
                cls.get_drmaa2_library().drmaa2_machineinfo_impl_spec())
        return cls.implementation_specific_keys

    @classmethod
    def to_py_machine_info_list(cls, ctypes_list):
        drmaa2_lib = cls.get_drmaa2_library()
        py_machine_info_list = list()
        if ctypes_list:
            count = drmaa2_lib.drmaa2_list_size(ctypes_list)
            cls.logger.debug('Converting ctypes machine info list of size {}'.format(count))
            for i in range(count):
                void_ptr = drmaa2_lib.drmaa2_list_get(ctypes_list, i)
                if void_ptr:
                    mi = cast(void_ptr, POINTER(drmaa2_machineinfo))
                    mi = MachineInfo(mi)
                    py_machine_info_list.append(mi)
                else:
                    ExceptionMapper.check_last_error_code()
                    # py_machine_info_list.append(None)
        return py_machine_info_list



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

from ctypes import POINTER
from ctypes import pointer

from .drmaa2_constants import *
from .drmaa2_ctypes import drmaa2_jinfo
from .drmaa2_object import Drmaa2Object
# from .drmaa2_slot_info_list_descriptor import Drmaa2SlotInfoListDescriptor
from .drmaa2_exceptions import InvalidArgument


class JobInfo(Drmaa2Object):
    """ High-level DRMAA2 job info class. """

    job_id = Drmaa2Object.StringDescriptor('jobId')
    """ Job id (str). """
    job_name = Drmaa2Object.StringDescriptor('jobName')
    """ Job name (str). """
    exit_status = Drmaa2Object.IntDescriptor('exitStatus')
    """ Job exit status (int). """
    terminating_signal = Drmaa2Object.StringDescriptor('terminatingSignal')
    """ Job terminating signal (str). """
    annotation = Drmaa2Object.StringDescriptor('annotation')
    """ Annotation (str). """
    job_state = Drmaa2Object.EnumDescriptor('jobState', JobState)
    """ Job state (JobState). """
    job_sub_state = Drmaa2Object.StringDescriptor('jobSubState')
    """ Job sub state (str). """
    allocated_machines = Drmaa2Object.StringListDescriptor('allocatedMachines')
    # allocated_machines = Drmaa2SlotInfoListDescriptor('allocatedMachines')
    """ List of machines allocated to the job ([str]). """
    submission_machine = Drmaa2Object.StringDescriptor('submissionMachine')
    """ Job submission machine (str). """
    job_owner = Drmaa2Object.StringDescriptor('jobOwner')
    """ Job owner (str). """
    slots = Drmaa2Object.LongLongDescriptor('slots')
    """ Number of job slots (long). """
    queue_name = Drmaa2Object.StringDescriptor('queueName')
    """ Queue name (str). """
    wallclock_time = Drmaa2Object.TimeDescriptor('wallclockTime')
    """ Wallclock time (long). """
    cpu_time = Drmaa2Object.LongLongDescriptor('cpuTime')
    """ CPU time (long). """
    submission_time = Drmaa2Object.TimeDescriptor('submissionTime')
    """ Submission time (datetime). """
    dispatch_time = Drmaa2Object.TimeDescriptor('dispatchTime')
    """ Dispatch time (datetime). """
    finish_time = Drmaa2Object.TimeDescriptor('finishTime')
    """ Finish time (datetime). """
    implementation_specific = Drmaa2Object.ImplSpecDescriptor('implementationSpecific')
    """ Implementation specific dictionary ({str:str}). """

    def __init__(self, job_info={}):
        """ 
        Constructor. 

        :param job_info: Input structure representing the object; this structure is typically a dictionary, but it can also be a low-level drmaa2_jinfo struct.
        :type job_info: dict 

        :raises InvalidArgument: in case of an invalid input argument.
        :raises Drmaa2Exception: for all other errors.

        >>> ji = JobInfo({'queue_name' : 'all.q', 'job_owner' : 'auser'})
        >>> print(ji.queue_name)
        all.q
        >>> ji.slots = 2
        >>> print(ji.slots)
        2
        """
        Drmaa2Object.__init__(self)
        if isinstance(job_info, dict):
            self._struct = POINTER(drmaa2_jinfo)()
            self._struct.contents = drmaa2_jinfo()
            # self._struct = self.get_drmaa2_library().drmaa2_jinfo_create()
            self.__init_defaults()
            self.init_impl_spec_key_values()
            self.from_dict(job_info)
        elif isinstance(job_info, POINTER(drmaa2_jinfo)):
            self._struct = job_info
        else:
            raise InvalidArgument('Invalid argument: %s' % str(job_info))

    def __del__(self):
        # self.get_drmaa2_library().drmaa2_jinfo_free(pointer(self._struct))
        pass

    def __init_defaults(self):
        self.job_id = None
        self.wallclock_time = UNSET_TIME
        self.exit_status = UNSET_NUM
        self.finish_time = UNSET_TIME
        self.job_state = JobState.UNSET_JSTATE
        self.submission_time = UNSET_TIME
        self.slots = UNSET_NUM
        self.dispatch_time = UNSET_TIME
        self.cpu_time = UNSET_NUM
        self.allocated_machines = None

    @classmethod
    def get_implementation_specific_keys(cls):
        """
        Retrieve list of implementation-specific keys.

        :returns: String list of implementation-specific keys.

        >>> print(JobInfo.get_implementation_specific_keys())
        ['uge_ji_priority', 'uge_ji_failed', 'uge_ji_mem', 'uge_ji_rss', 'uge_ji_vmem']
        """
        if cls.implementation_specific_keys is None:
            cls.implementation_specific_keys = cls.to_py_string_list(cls.get_drmaa2_library().drmaa2_jinfo_impl_spec())
        return cls.implementation_specific_keys



#!/usr/bin/env python
# ___INFO__MARK_BEGIN__
########################################################################## 
# Copyright 2016-2019 Univa Corporation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0 
# 
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, 
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
# See the License for the specific language governing permissions and 
# limitations under the License. 
########################################################################### 
# ___INFO__MARK_END__

"""
Definitions for DRMAA2 ctypes.
"""

import ctypes
import platform
from ctypes import c_void_p
from ctypes import c_byte
from ctypes import c_char
from ctypes import c_char_p
from ctypes import c_long
from ctypes import c_int
from ctypes import c_longlong
from ctypes import c_float
from ctypes import Structure
from ctypes import POINTER
from ctypes import CFUNCTYPE

DRMAA2_CHAR_BUFFER_SIZE = 128

# Basic types.
drmaa2_time = c_longlong
drmaa2_enum = c_int
drmaa2_bool = drmaa2_enum
drmaa2_capability = drmaa2_enum
drmaa2_cpu = drmaa2_enum
drmaa2_error = drmaa2_enum
drmaa2_event = drmaa2_enum
drmaa2_listtype = drmaa2_enum
drmaa2_os = drmaa2_enum
drmaa2_jstate = drmaa2_enum


# drmaa2_string must be a subclass of c_char_p to allow us
# to free DRMAA2 strings.
class drmaa2_string(c_char_p):
    python_version = platform.python_version_tuple()[0]

    #    def __init__(self, **kwargs):
    #        """
    #        Ctypes.Structure with integrated default values.
    #
    #        :param kwargs: values different to defaults
    #        :type kwargs: dict
    #        """
    #        values = type(self)._defaults_.copy()
    #        for (key, val) in kwargs.items():
    #            values[key] = val
    #        super().__init__(**values)

    def __eq__(self, other):
        if isinstance(other, str):
            if self.python_version == '2':
                return self.value == other
            else:
                return self.value == other.encode()
        else:
            return self.value == other.value

    def __ne__(self, other):
        return not self.__eq__(other)

    def encode(self):
        return self.value

    def decode(self):
        return self.value


# List types
drmaa2_list = c_void_p
drmaa2_string_list = drmaa2_list
drmaa2_j_list = drmaa2_list
drmaa2_queueinfo_list = drmaa2_list
drmaa2_machineinfo_list = drmaa2_list
drmaa2_slotinfo_list = drmaa2_list
drmaa2_r_list = drmaa2_list
drmaa2_list_entryfree = CFUNCTYPE(None, POINTER(c_void_p))

# Dict type
drmaa2_dict = c_void_p
drmaa2_dict_entryfree = CFUNCTYPE(None, POINTER(c_char_p), POINTER(c_char_p))


# DRMAA2 structs.
class drmaa2_struct(Structure):
    def __eq__(self, other):
        result = True
        for (attr_name, _) in self._fields_:
            a, b = getattr(self, attr_name), getattr(other, attr_name)
            if isinstance(a, ctypes.Array):
                if a[:] != b[:]:
                    result = False
                    break
            elif a != b:
                result = False
                break
        return result

    def __ne__(self, other):
        return not self.__eq__(other)


class drmaa2_sudo(drmaa2_struct):
    _fields_ = [("username", c_char * DRMAA2_CHAR_BUFFER_SIZE),
                ("groupname", c_char * DRMAA2_CHAR_BUFFER_SIZE),
                ("uid", c_long),
                ("gid", c_long)]


class drmaa2_jinfo(drmaa2_struct):
    _fields_ = [("jobId", drmaa2_string),
                ("jobName", drmaa2_string),
                ("exitStatus", c_int),
                ("terminatingSignal", drmaa2_string),
                ("annotation", drmaa2_string),
                ("jobState", drmaa2_jstate),
                ("jobSubState", drmaa2_string),
                ("allocatedMachines", drmaa2_slotinfo_list),
                ("submissionMachine", drmaa2_string),
                ("jobOwner", drmaa2_string),
                ("slots", c_longlong),
                ("queueName", drmaa2_string),
                ("wallclockTime", drmaa2_time),
                ("cpuTime", c_longlong),
                ("submissionTime", drmaa2_time),
                ("dispatchTime", drmaa2_time),
                ("finishTime", drmaa2_time),
                ("implementationSpecific", c_void_p)]


class drmaa2_slotinfo(drmaa2_struct):
    _fields_ = [("machineName", drmaa2_string),
                ("slots", c_longlong),
                ("implementationSpecific", c_void_p)]


class drmaa2_rinfo(drmaa2_struct):
    _fields_ = [("reservationId", drmaa2_string),
                ("reservationName", drmaa2_string),
                ("reservedStartTime", drmaa2_time),
                ("reservedEndTime", drmaa2_time),
                ("usersACL", drmaa2_string_list),
                ("reservedSlots", c_longlong),
                ("reservedMachines", drmaa2_slotinfo_list),
                ("implementationSpecific", c_void_p)]


class drmaa2_jtemplate(drmaa2_struct):
    _fields_ = [("remoteCommand", drmaa2_string),
                ("args", drmaa2_string_list),
                ("submitAsHold", drmaa2_bool),
                ("rerunnable", drmaa2_bool),
                ("jobEnvironment", drmaa2_dict),
                ("workingDirectory", drmaa2_string),
                ("jobCategory", drmaa2_string),
                ("email", drmaa2_string_list),
                ("emailOnStarted", drmaa2_bool),
                ("emailOnTerminated", drmaa2_bool),
                ("jobName", drmaa2_string),
                ("inputPath", drmaa2_string),
                ("outputPath", drmaa2_string),
                ("errorPath", drmaa2_string),
                ("joinFiles", drmaa2_bool),
                ("reservationId", drmaa2_string),
                ("queueName", drmaa2_string),
                ("minSlots", c_longlong),
                ("maxSlots", c_longlong),
                ("priority", c_longlong),
                ("candidateMachines", drmaa2_string_list),
                ("minPhysMemory", c_longlong),
                ("machineOS", drmaa2_os),
                ("machineArch", drmaa2_cpu),
                ("startTime", drmaa2_time),
                ("deadlineTime", drmaa2_time),
                ("stageInFiles", drmaa2_dict),
                ("stageOutFiles", drmaa2_dict),
                ("resourceLimits", drmaa2_dict),
                ("accountingId", drmaa2_string),
                ("implementationSpecific", c_void_p)]


class drmaa2_rtemplate(drmaa2_struct):
    _fields_ = [("reservationName", drmaa2_string),
                ("startTime", drmaa2_time),
                ("endTime", drmaa2_time),
                ("duration", drmaa2_time),
                ("minSlots", c_longlong),
                ("maxSlots", c_longlong),
                ("jobCategory", drmaa2_string),
                ("usersACL", drmaa2_string_list),
                ("candidateMachines", drmaa2_string_list),
                ("minPhysMemory", c_longlong),
                ("machineOS", drmaa2_os),
                ("machineArch", drmaa2_cpu),
                ("implementationSpecific", c_void_p)]


class drmaa2_notification(drmaa2_struct):
    _fields_ = [("event", drmaa2_event),
                ("jobId", drmaa2_string),
                ("sessionName", drmaa2_string),
                ("jobState", drmaa2_jstate),
                ("implementationSpecific", c_void_p)]


class drmaa2_queueinfo(drmaa2_struct):
    _fields_ = [("name", drmaa2_string),
                ("implementationSpecific", c_void_p)]


class drmaa2_version(drmaa2_struct):
    _fields_ = [("major", drmaa2_string),
                ("minor", drmaa2_string),
                ("implementationSpecific", c_void_p)]


class drmaa2_machineinfo(drmaa2_struct):
    _fields_ = [("name", drmaa2_string),
                ("available", drmaa2_bool),
                ("sockets", c_longlong),
                ("coresPerSocket", c_longlong),
                ("threadsPerCore", c_longlong),
                ("load", c_float),
                ("physMemory", c_longlong),
                ("virtMemory", c_longlong),
                ("machineArch", drmaa2_cpu),
                ("machineOSVersion", POINTER(drmaa2_version)),
                ("machineOS", drmaa2_os),
                ("implementationSpecific", c_void_p)]


# Callback function
drmaa2_callback = CFUNCTYPE(None, POINTER(drmaa2_notification))


# UGE-specific structs
class drmaa2_j(drmaa2_struct):
    _fields_ = [("id", drmaa2_string),
                ("session_name", drmaa2_string),
                ("job_name", drmaa2_string)]


class drmaa2_jarray(drmaa2_struct):
    _fields_ = [("id", drmaa2_string),
                ("job_list", drmaa2_j_list),
                ("session_name", drmaa2_string)]


class drmaa2_jsession(drmaa2_struct):
    _fields_ = [("contact", drmaa2_string),
                ("name", drmaa2_string)]


class drmaa2_msession(drmaa2_struct):
    _fields_ = [("name", drmaa2_string)]


class drmaa2_r(drmaa2_struct):
    _fields_ = [("id", drmaa2_string),
                ("session_name", drmaa2_string)]


class drmaa2_rsession(drmaa2_struct):
    _fields_ = [("contact", drmaa2_string),
                ("name", drmaa2_string)]


#######################################################################
# Test.
if __name__ == '__main__':
    pass
